@@ -7,7 +7,7 @@
 //----------------------------------------------------------------
 
 #include "getPsi.h"
-#include "samplePsi.h"
+#include "bootstrapCI.h"
 #include "exactdistribution.h"
 #include <R_ext/Rdynload.h>
 //----------------------------------------------------------------
@@ -15,7 +15,7 @@
 static const R_CallMethodDef callMethods[] = {
 {"getPsi202",  (DL_FUNC) &getPsi202, 1},
 {"exactDistr202",(DL_FUNC) &exactDistr202,2},
-{"samplePsi",(DL_FUNC) &samplePsi,5},
+{"bootstrapCI",(DL_FUNC) &bootstrapCI,4},
 {NULL, NULL, 0}
 };
 //----------------------------------------------------------------
